@@ -22,22 +22,22 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from .enum import Enum
-from .enum import unique
+from enum import Enum
+from enum import unique
+import warnings
+
 from .utils import create_instance
 from .utils import positive_int_value
 from pyeclib_c import get_liberasurecode_version
 
-import warnings
-
 
 def check_backend_available(backend_name):
     from pyeclib_c import check_backend_available
 
     if backend_name.startswith('flat_xor_hd'):
-        int_type = PyECLib_EC_Types.get_by_name('flat_xor_hd')
+        int_type = PyECLib_EC_Types['flat_xor_hd']
     else:
-        int_type = PyECLib_EC_Types.get_by_name(backend_name)
+        int_type = PyECLib_EC_Types[backend_name]
     if not int_type:
         return False
     return check_backend_available(int_type.value)
@@ -48,54 +48,20 @@ def PyECLibVersion(z, y, x):
 
 
 PYECLIB_MAJOR = 1
-PYECLIB_MINOR = 6
-PYECLIB_REV = 4
+PYECLIB_MINOR = 7
+PYECLIB_REV = 0
 PYECLIB_VERSION = PyECLibVersion(PYECLIB_MAJOR, PYECLIB_MINOR,
                                  PYECLIB_REV)
+__version__ = '%d.%d.%d' % (PYECLIB_MAJOR, PYECLIB_MINOR, PYECLIB_REV)
 
 
 PYECLIB_MAX_DATA = 32
 PYECLIB_MAX_PARITY = 32
 
 
-@unique
-class PyECLibEnum(Enum):
-
-    def describe(self):
-        # returns supported types
-        return list(self)
-
-    @classmethod
-    def has_enum(cls, name):
-        # returns True if name is a valid member of the enum
-        try:
-            cls.__getattr__(name)
-        except AttributeError:
-            return False
-        return True
-
-    @classmethod
-    def get_by_name(cls, name):
-        try:
-            obj = cls.__getattr__(name)
-        except AttributeError:
-            return None
-        return obj
-
-    @classmethod
-    def names(cls):
-        return [name for name, value in cls.__members__.items()]
-
-    @classmethod
-    def values(cls):
-        return [value for name, value in cls.__members__.items()]
-
-    def __str__(self):
-        return "%s: %d" % (self.name, self.value)
-
-
 # Erasure Code backends supported as of this PyECLib API rev
-class PyECLib_EC_Types(PyECLibEnum):
+@unique
+class PyECLib_EC_Types(Enum):
     # Note: the Enum start value defaults to 1 as the starting value and not 0
     # 0 is False in the boolean sense but enum members evaluate to True
     jerasure_rs_vand = 1
@@ -106,6 +72,7 @@ class PyECLib_EC_Types(PyECLibEnum):
     liberasurecode_rs_vand = 6
     isa_l_rs_cauchy = 7
     libphazr = 8
+    isa_l_rs_vand_inv = 9
 
 
 # Output of Erasure (en)Coding process are data "fragments".  Fragment data
@@ -114,7 +81,8 @@ class PyECLib_EC_Types(PyECLibEnum):
 
 # The following Enum defines the schemes supported for fragment checksums.
 # The checksum type is "none" unless specified.
-class PyECLib_FRAGHDRCHKSUM_Types(PyECLibEnum):
+@unique
+class PyECLib_FRAGHDRCHKSUM_Types(Enum):
     # Note: the Enum start value defaults to 1 as the starting value and not 0
     # 0 is False in the boolean sense but enum members evaluate to True
     none = 1
@@ -125,7 +93,15 @@ class PyECLib_FRAGHDRCHKSUM_Types(PyECLibEnum):
 class ECDriver(object):
     '''A driver to encode, decode, and reconstruct erasure-coded data.'''
 
-    def __init__(self, **kwargs):
+    def __init__(
+        self, *,
+        ec_type=None,
+        library_import_str='pyeclib.core.ECPyECLibDriver',
+        k,
+        m,
+        chksum_type='none',
+        validate=False,
+    ):
         '''
         :param ec_type: the erasure coding type to use for this driver.
         :param k: number of data fragments to use. Required.
@@ -144,68 +120,63 @@ class ECDriver(object):
         self.hd = -1
         self.ec_type = None
         self.chksum_type = None
-        self.validate = False
 
-        for required in ('k', 'm'):
-            if required not in kwargs:
-                raise ECDriverError(
-                    "Invalid Argument: %s is required" % required)
-
-        if 'ec_type' not in kwargs and 'library_import_str' not in kwargs:
+        if (
+            ec_type is None and
+            library_import_str == 'pyeclib.core.ECPyECLibDriver'
+        ):
             raise ECDriverError(
                 "Invalid Argument: either ec_type or library_import_str "
                 "must be provided")
 
-        for (key, value) in kwargs.items():
-            if key == "k":
-                try:
-                    self.k = positive_int_value(value)
-                except ValueError:
-                    raise ECDriverError(
-                        "Invalid number of data fragments (k)")
-            elif key == "m":
-                try:
-                    self.m = positive_int_value(value)
-                except ValueError:
-                    raise ECDriverError(
-                        "Invalid number of parity fragments (m)")
-            elif key == "ec_type":
-                if value in ["flat_xor_hd", "flat_xor_hd_3", "flat_xor_hd_4"]:
-                    if value == "flat_xor_hd" or value == "flat_xor_hd_3":
-                        self.hd = 3
-                    elif value == "flat_xor_hd_4":
-                        self.hd = 4
-                    value = "flat_xor_hd"
-                elif value == "libphazr":
-                    self.hd = 1
+        try:
+            self.k = positive_int_value(k)
+        except ValueError:
+            raise ECDriverError(
+                "Invalid number of data fragments (k)")
 
-                if PyECLib_EC_Types.has_enum(value):
-                    self.ec_type = PyECLib_EC_Types.get_by_name(value)
-                    if self.ec_type in (PyECLib_EC_Types.jerasure_rs_vand,
-                                        PyECLib_EC_Types.jerasure_rs_cauchy):
-                        warnings.warn('Jerasure support is deprecated and '
-                                      'may be removed in a future release',
-                                      FutureWarning, stacklevel=2)
+        try:
+            self.m = positive_int_value(m)
+        except ValueError:
+            raise ECDriverError(
+                "Invalid number of parity fragments (m)")
 
-                else:
-                    raise ECBackendNotSupported(
-                        "%s is not a valid EC type for PyECLib!" % value)
-            elif key == "chksum_type":
-                if PyECLib_FRAGHDRCHKSUM_Types.has_enum(value):
-                    self.chksum_type = \
-                        PyECLib_FRAGHDRCHKSUM_Types.get_by_name(value)
-                else:
-                    raise ECDriverError(
-                        "%s is not a valid checksum type for PyECLib!" % value)
-            elif key == "validate":
-                # validate if the ec type is available (runtime check)
-                self.validate = value
+        if ec_type:
+            if ec_type in ["flat_xor_hd", "flat_xor_hd_3", "flat_xor_hd_4"]:
+                if ec_type == "flat_xor_hd" or ec_type == "flat_xor_hd_3":
+                    self.hd = 3
+                elif ec_type == "flat_xor_hd_4":
+                    self.hd = 4
+                ec_type = "flat_xor_hd"
+            elif ec_type == "libphazr":
+                self.hd = 1
+
+            if ec_type in PyECLib_EC_Types.__members__:
+                self.ec_type = PyECLib_EC_Types[ec_type]
+                if self.ec_type in (PyECLib_EC_Types.jerasure_rs_vand,
+                                    PyECLib_EC_Types.jerasure_rs_cauchy):
+                    warnings.warn('Jerasure support is deprecated and '
+                                  'may be removed in a future release',
+                                  FutureWarning, stacklevel=2)
+
+            else:
+                raise ECBackendNotSupported(
+                    "%s is not a valid EC type for PyECLib!" % ec_type)
+
+        if chksum_type in PyECLib_FRAGHDRCHKSUM_Types.__members__:
+            self.chksum_type = \
+                PyECLib_FRAGHDRCHKSUM_Types[chksum_type]
+        else:
+            raise ECDriverError(
+                "%s is not a valid checksum type for PyECLib!"
+                % chksum_type)
+
+        self.validate = validate
 
         if self.hd == -1:
             self.hd = self.m
 
-        self.library_import_str = kwargs.pop('library_import_str',
-                                             'pyeclib.core.ECPyECLibDriver')
+        self.library_import_str = library_import_str
         #
         # Instantiate EC backend driver
         #
@@ -243,12 +214,15 @@ class ECDriver(object):
                 "in %s: %s" % (self.library_import_str, missing_methods))
 
     def __repr__(self):
-        return '%s(ec_type=%r, k=%r, m=%r)' % (
-            type(self).__name__,
-            'flat_xor_hd_%s' % self.hd if self.ec_type.name == 'flat_xor_hd'
-            else self.ec_type.name,
-            self.k,
-            self.m)
+        if self.ec_type is None:
+            ec_type = 'None'
+        elif self.ec_type.name == 'flat_xor_hd':
+            ec_type = f'flat_xor_hd_{self.hd}'
+        else:
+            ec_type = self.ec_type.name
+
+        return (f'{type(self).__name__}('
+                f'ec_type={ec_type!r}, k={self.k}, m={self.m})')
 
     def close(self):
         self.ec_lib_reference.close()
@@ -345,7 +319,7 @@ class ECDriver(object):
         Verify a subset of fragments generated by encode()
 
         :param fragment_metadata_list: a list of buffers representing the
-                                       metadata from a subset of the framgments
+                                       metadata from a subset of the fragments
                                        generated by encode().
         :returns: 'None' if the metadata is consistent.
                   a list of fragment indexes corresponding to inconsistent
@@ -531,6 +505,7 @@ ALL_EC_TYPES = [
     'liberasurecode_rs_vand',
     'isa_l_rs_cauchy',
     'libphazr',
+    'isa_l_rs_vand_inv',
 ]
 
 
