@@ -35,33 +35,19 @@
 
 #include <pyeclib_c.h>
 
-/* Python 3 compatibility macros */
-#if PY_MAJOR_VERSION >= 3
-  #define PYTHON3
-  #define MOD_ERROR_VAL NULL
-  #define MOD_SUCCESS_VAL(val) val
-  #define MOD_INIT(name) PyMODINIT_FUNC PyInit_##name(void)
-  #define MOD_DEF(ob, name, doc, methods) \
-          static struct PyModuleDef moduledef = { \
-              PyModuleDef_HEAD_INIT, name, doc, -1, methods, }; \
-          ob = PyModule_Create(&moduledef);
-  #define PY_BUILDVALUE_OBJ_LEN(obj, objlen) \
-          Py_BuildValue("y#", obj, (Py_ssize_t)objlen)
-  #define PyInt_FromLong PyLong_FromLong
-  #define PyString_FromString PyUnicode_FromString
-  #define ENCODE_ARGS "Oy#"
-  #define GET_METADATA_ARGS "Oy#i"
-#else
-  #define MOD_ERROR_VAL
-  #define MOD_SUCCESS_VAL(val)
-  #define MOD_INIT(name) void init##name(void)
-  #define MOD_DEF(ob, name, doc, methods) \
-          ob = Py_InitModule3(name, methods, doc);
-  #define PY_BUILDVALUE_OBJ_LEN(obj, objlen) \
-          Py_BuildValue("s#", obj, (Py_ssize_t)objlen)
-  #define ENCODE_ARGS "Os#"
-  #define GET_METADATA_ARGS "Os#i"
-#endif
+#define MOD_ERROR_VAL NULL
+#define MOD_SUCCESS_VAL(val) val
+#define MOD_INIT(name) PyMODINIT_FUNC PyInit_##name(void)
+#define MOD_DEF(ob, name, doc, methods) \
+      static struct PyModuleDef moduledef = { \
+          PyModuleDef_HEAD_INIT, name, doc, -1, methods, }; \
+      ob = PyModule_Create(&moduledef);
+#define PY_BUILDVALUE_OBJ_LEN(obj, objlen) \
+      Py_BuildValue("y#", obj, (Py_ssize_t)objlen)
+#define PyInt_FromLong PyLong_FromLong
+#define PyString_FromString PyUnicode_FromString
+#define ENCODE_ARGS "Oy#"
+#define GET_METADATA_ARGS "Oy#i"
 
 
 typedef struct pyeclib_byte_range {
@@ -275,12 +261,10 @@ pyeclib_c_init(PyObject *self, PyObject *args)
   pyeclib_obj_handle = PyCapsule_New(pyeclib_handle, PYECC_HANDLE_NAME,
                                      pyeclib_c_destructor);
 
-  /* Clean up the allocated memory on error, or update the ref count */
+  /* Clean up the allocated memory on error */
   if (pyeclib_obj_handle == NULL) {
     pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_init");
     goto cleanup;
-  } else {
-    Py_INCREF(pyeclib_obj_handle);
   }
 
 exit:
@@ -420,8 +404,8 @@ pyeclib_c_get_segment_info(PyObject *self, PyObject *args)
   /* The minimum segment size depends on the EC algorithm */
   min_segment_size = liberasurecode_get_minimum_encode_size(pyeclib_handle->ec_desc);
   if (min_segment_size < 0) {
-      pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
-      return NULL;
+    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
+    return NULL;
   }
 
   /* Get the number of segments */
@@ -449,8 +433,8 @@ pyeclib_c_get_segment_info(PyObject *self, PyObject *args)
 
     fragment_size = liberasurecode_get_fragment_size(pyeclib_handle->ec_desc, data_len);
     if (fragment_size < 0) {
-        pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
-	return NULL;
+      pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
+      return NULL;
     }
 
     /* Segment size is the user-provided segment size */
@@ -465,8 +449,8 @@ pyeclib_c_get_segment_info(PyObject *self, PyObject *args)
 
     fragment_size = liberasurecode_get_fragment_size(pyeclib_handle->ec_desc, segment_size);
     if (fragment_size < 0) {
-        pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
-	return NULL;
+      pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
+      return NULL;
     }
 
     last_segment_size = data_len - (segment_size * (num_segments - 1));
@@ -485,8 +469,8 @@ pyeclib_c_get_segment_info(PyObject *self, PyObject *args)
 
     last_fragment_size = liberasurecode_get_fragment_size(pyeclib_handle->ec_desc, last_segment_size);
     if (fragment_size < 0) {
-        pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
-	return NULL;
+      pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_get_segment_info");
+      return NULL;
     }
   }
 
@@ -503,19 +487,11 @@ pyeclib_c_get_segment_info(PyObject *self, PyObject *args)
     "last_fragment_size", last_fragment_size,
     "num_segments", num_segments);
   if (NULL == ret_dict) {
-    goto error;
+    pyeclib_c_seterr(-ENOMEM, "pyeclib_c_get_segment_info");
+    return NULL;
   }
 
-exit:
-    return ret_dict;
-
-error:
-    // To prevent unexpected call, this is placed after return call
-    pyeclib_c_seterr(-ENOMEM, "pyeclib_c_get_segment_info");
-    Py_XDECREF(ret_dict);
-    ret_dict = NULL;
-    goto exit;
-
+  return ret_dict;
 }
 
 
@@ -858,20 +834,12 @@ pyeclib_c_decode(PyObject *self, PyObject *args)
 
         if (PyLong_Check(py_begin))
             begin = PyLong_AsLong(py_begin);
-#ifndef PYTHON3
-        else if (PyInt_Check(py_begin))
-            begin = PyInt_AsLong(py_begin);
-#endif
         else {
           pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_decode invalid range");
           goto error;
         }
         if (PyLong_Check(py_end))
             end = PyLong_AsLong(py_end);
-#ifndef PYTHON3
-        else if (PyInt_Check(py_end))
-            end = PyInt_AsLong(py_end);
-#endif
         else {
           pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_decode invalid range");
           goto error;
@@ -1010,6 +978,9 @@ static const char* backend_id_to_str(uint8_t backend_id)
     case 4:
       backend_id_str = "isa_l_rs_vand\0";
       break;
+    case 5:
+      backend_id_str = "shss\0";
+      break;
     case 6:
       backend_id_str = "liberasurecode_rs_vand\0";
       break;
@@ -1019,6 +990,9 @@ static const char* backend_id_to_str(uint8_t backend_id)
     case 8:
       backend_id_str = "libphazr\0";
       break;
+    case 9:
+      backend_id_str = "isa_l_rs_vand_inv\0";
+      break;
     default:
       backend_id_str = "unknown\0";
   }
@@ -1145,19 +1119,19 @@ pyeclib_c_check_metadata(PyObject *self, PyObject *args)
 
   /* Obtain and validate the method parameters */
   if (!PyArg_ParseTuple(args, "OO", &pyeclib_obj_handle, &fragment_metadata_list)) {
-    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_encode");
+    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_check_metadata");
     return NULL;
   }
   pyeclib_handle = (pyeclib_t*)PyCapsule_GetPointer(pyeclib_obj_handle, PYECC_HANDLE_NAME);
   if (pyeclib_handle == NULL) {
-    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_encode");
+    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_check_metadata");
     return NULL;
   }
   k = pyeclib_handle->ec_args.k;
   m = pyeclib_handle->ec_args.m;
   num_fragments = k + m;
   if (num_fragments != PyList_Size(fragment_metadata_list)) {
-    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_encode");
+    pyeclib_c_seterr(-EINVALIDPARAMS, "pyeclib_c_check_metadata");
     return NULL;
   }
 
@@ -1165,7 +1139,7 @@ pyeclib_c_check_metadata(PyObject *self, PyObject *args)
   size = sizeof(char * ) * num_fragments;
   c_fragment_metadata_list = (char **) alloc_zeroed_buffer(size);
   if (NULL == c_fragment_metadata_list) {
-    pyeclib_c_seterr(-ENOMEM, "pyeclib_c_encode");
+    pyeclib_c_seterr(-ENOMEM, "pyeclib_c_check_metadata");
     goto error;
   }
 
@@ -1179,15 +1153,20 @@ pyeclib_c_check_metadata(PyObject *self, PyObject *args)
   ret = liberasurecode_verify_stripe_metadata(pyeclib_handle->ec_desc, c_fragment_metadata_list,
                                               num_fragments);
 
-  ret_obj = PyDict_New();
   if (ret == 0) {
+    ret_obj = PyDict_New();
+    if (NULL == ret_obj) {
+      pyeclib_c_seterr(-ENOMEM, "pyeclib_c_check_metadata");
+      goto error;
+    }
     PyDict_SetItemString(ret_obj, "status", PyLong_FromLong((long)0));
-  } else if (ret == -EINVALIDPARAMS) {
-    PyDict_SetItemString(ret_obj, "status", PyLong_FromLong((long)-EINVALIDPARAMS));
-    PyDict_SetItemString(ret_obj, "reason", PyString_FromString("Invalid arguments"));
-    goto error;
   } else if (ret == -EBADCHKSUM) {
-    PyDict_SetItemString(ret_obj, "status", PyLong_FromLong((long)-EINVALIDPARAMS));
+    ret_obj = PyDict_New();
+    if (NULL == ret_obj) {
+      pyeclib_c_seterr(-ENOMEM, "pyeclib_c_check_metadata");
+      goto error;
+    }
+    PyDict_SetItemString(ret_obj, "status", PyLong_FromLong((long)ret));
     PyDict_SetItemString(ret_obj, "reason", PyString_FromString("Bad checksum"));
     PyObject *bad_chksums = PyList_New(0);
     for (i = 0; i < num_fragments; i++) {
@@ -1197,14 +1176,11 @@ pyeclib_c_check_metadata(PyObject *self, PyObject *args)
       }
     }
     PyDict_SetItemString(ret_obj, "bad_fragments", bad_chksums);
+  } else {
+    pyeclib_c_seterr(ret, "pyeclib_c_check_metadata");
   }
 
-  goto exit;
-
 error:
-  ret_obj = NULL;
-
-exit:
   free(c_fragment_metadata_list);
 
   return ret_obj;
