@@ -22,23 +22,18 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import os
-try:
-    import queue
-except ImportError:  # py2
-    import Queue as queue
+import queue
 import random
 import resource
 import string
-import sys
 import tempfile
 import threading
 import unittest
 
 from itertools import combinations
 
-import six
-
 import pyeclib.ec_iface
+from pyeclib.ec_iface import ECBackendInstanceNotAvailable
 from pyeclib.ec_iface import ECBackendNotSupported
 from pyeclib.ec_iface import ECDriver
 from pyeclib.ec_iface import ECDriverError
@@ -51,14 +46,6 @@ from pyeclib.ec_iface import ALL_EC_TYPES
 from pyeclib.ec_iface import VALID_EC_TYPES
 
 
-if sys.version < '3':
-    def b2i(b):
-        return ord(b)
-else:
-    def b2i(b):
-        return b
-
-
 class TestNullDriver(unittest.TestCase):
 
     def setUp(self):
@@ -112,8 +99,7 @@ class TestPyECLibDriver(unittest.TestCase):
             # Create the dictionary of files to test with
             buf = ''.join(random.choice(string.ascii_letters)
                           for i in range(size))
-            if sys.version_info >= (3,):
-                buf = buf.encode('ascii')
+            buf = buf.encode('ascii')
             tmp_file = tempfile.NamedTemporaryFile()
             tmp_file.write(buf)
             cls.files[size_str] = tmp_file
@@ -138,16 +124,16 @@ class TestPyECLibDriver(unittest.TestCase):
 
         for ec_type in VALID_EC_TYPES:
             # missing k
-            with self.assertRaises(ECDriverError) as err_context:
+            with self.assertRaises(TypeError) as err_context:
                 ECDriver(ec_type=ec_type, m=1)
-            self.assertEqual(str(err_context.exception),
-                             "Invalid Argument: k is required")
+            self.assertIn("missing 1 required keyword-only argument: 'k'",
+                          str(err_context.exception))
 
             # missing m
-            with self.assertRaises(ECDriverError) as err_context:
+            with self.assertRaises(TypeError) as err_context:
                 ECDriver(ec_type=ec_type, k=1)
-            self.assertEqual(str(err_context.exception),
-                             "Invalid Argument: m is required")
+            self.assertIn("missing 1 required keyword-only argument: 'm'",
+                          str(err_context.exception))
 
     def test_invalid_km_args(self):
         for ec_type in VALID_EC_TYPES:
@@ -294,8 +280,42 @@ class TestPyECLibDriver(unittest.TestCase):
         if _type7 in VALID_EC_TYPES:
             pyeclib_drivers.append(ECDriver(k=4, m=4, ec_type=_type7,
                                    chksum_type=csum))
+
+        _type8 = 'isa_l_rs_vand_inv'
+        if _type8 in VALID_EC_TYPES:
+            pyeclib_drivers.append(ECDriver(k=12, m=2, ec_type=_type8,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=11, m=2, ec_type=_type8,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=10, m=2, ec_type=_type8,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=8, m=4, ec_type=_type8,
+                                   chksum_type=csum))
+            pyeclib_drivers.append(ECDriver(k=11, m=7, ec_type=_type8,
+                                   chksum_type=csum))
         return pyeclib_drivers
 
+    def test_use_after_close(self):
+        pyeclib_drivers = self.get_pyeclib_testspec()
+        for pyeclib_driver in pyeclib_drivers:
+            frags = pyeclib_driver.encode(b"testdata")
+            self.assertEqual(
+                pyeclib_driver.reconstruct(frags[1:], [0])[0], frags[0])
+
+            pyeclib_driver.close()
+            with self.assertRaises(ECBackendInstanceNotAvailable) as ctx:
+                pyeclib_driver.encode(b"testdata")
+            self.assertEqual(str(ctx.exception),
+                             'erasure coding handle is closed')
+            with self.assertRaises(ECBackendInstanceNotAvailable) as ctx:
+                pyeclib_driver.decode(frags)
+            self.assertEqual(str(ctx.exception),
+                             'erasure coding handle is closed')
+            with self.assertRaises(ECBackendInstanceNotAvailable) as ctx:
+                pyeclib_driver.reconstruct(frags[1:], [0])
+            self.assertEqual(str(ctx.exception),
+                             'erasure coding handle is closed')
+
     def test_small_encode(self):
         pyeclib_drivers = self.get_pyeclib_testspec()
         encode_strs = [b"a", b"hello", b"hellohyhi", b"yo"]
@@ -333,7 +353,7 @@ class TestPyECLibDriver(unittest.TestCase):
         self.assertGreater(len(pyeclib_drivers), 0)
         pyeclib_driver = pyeclib_drivers[0]
 
-        from six.moves import builtins
+        import builtins
         real_import = builtins.__import__
 
         def fake_import(*a, **kw):
@@ -471,7 +491,7 @@ class TestPyECLibDriver(unittest.TestCase):
             for fragment in fragments:
                 if i in fragments_to_corrupt:
                     corrupted_fragment = fragment[:100] +\
-                        (str(chr((b2i(fragment[100]) + 0x1)
+                        (str(chr((fragment[100] + 0x1)
                                  % 128))).encode('utf-8') + fragment[101:]
                     fragment_metadata_list.append(
                         pyeclib_driver.get_metadata(corrupted_fragment))
@@ -480,7 +500,7 @@ class TestPyECLibDriver(unittest.TestCase):
                         pyeclib_driver.get_metadata(fragment))
                 i += 1
 
-            expected_ret_value = {"status": -206,
+            expected_ret_value = {"status": -205,
                                   "reason": "Bad checksum",
                                   "bad_fragments": fragments_to_corrupt}
             self.assertEqual(
@@ -625,7 +645,7 @@ class TestPyECLibDriver(unittest.TestCase):
         # and if you want to test either other parameters or backends,
         # you can add the spec you want to test there.
         pyeclib_drivers = self.get_pyeclib_testspec()
-        orig_data = os.urandom(1024 ** 2)
+        orig_data = os.urandom(1024)
         for pyeclib_driver in pyeclib_drivers:
             encoded = pyeclib_driver.encode(orig_data)
             # make all fragment like (index, frag_data) format to feed
@@ -836,8 +856,8 @@ class BackendsEnabledMetaclass(type):
         return type.__new__(meta, cls_name, cls_bases, cls_dict)
 
 
-class TestBackendsEnabled(six.with_metaclass(BackendsEnabledMetaclass,
-                                             unittest.TestCase)):
+class TestBackendsEnabled(unittest.TestCase,
+                          metaclass=BackendsEnabledMetaclass):
     '''
     Based on TestPyECLibDriver.test_valid_algo above, but these tests
     should *always* either pass or skip.
